@@ -591,6 +591,9 @@ void printAddressProperties(std::ostream&         os,
       else if(IN6_IS_ADDR_V4MAPPED(&ipv6address)) {
          os << "   - IPv4-mapped IPv6 address" << std::endl;
       }
+      else if(hasTranslationPrefix(&address.in6)) {
+         os << "   - IPv4-embedded IPv6 address" << std::endl;
+      }
 
       // ------ Multicast addresses -----------------------------------------
       else if(IN6_IS_ADDR_MULTICAST(&ipv6address)) {
@@ -691,7 +694,7 @@ void printAddressProperties(std::ostream&         os,
             const uint32_t u = ntohs(((const uint16_t*)&ipv6address.s6_addr)[1]);
             const uint32_t l = ntohs(((const uint16_t*)&ipv6address.s6_addr)[2]);
             sixToFour.in.sin_addr.s_addr = htonl((u << 16) | l);
-            printAddress(std::cerr, &sixToFour.sa, false);
+            printAddress(std::cout, &sixToFour.sa, false);
             os << std::endl;
          }
 
@@ -867,6 +870,11 @@ int main(int argc, char** argv)
       }
    }
    std::cout << "Wildcard Mask = " << wildcard << std::endl;
+   if(isIPv4(address)) {
+      char hex[16];
+      snprintf((char*)&hex, sizeof(hex), "%08X", ntohl(address.in.sin_addr.s_addr));
+      std::cout << "Hex. Address  = " << hex << std::endl;
+   }
    std::cout << "Hosts Bits    = " << hostBits << std::endl;
    if(!isMulticast(address)) {
       char maxHostsString[128];
